@@ -23,6 +23,7 @@ class Transformer(object):
             embedding_size=None,  # 是否指定embedding_size
             keep_tokens=None,  # 要保留的词ID列表
             layers=None,  # 外部传入的Keras层
+            name=None,  # 模型名称
             **kwargs):
         if keep_tokens is None:
             self.vocab_size = vocab_size
@@ -40,6 +41,7 @@ class Transformer(object):
         self.attention_mask = None
         self.position_bias = None
         self.layers = {} if layers is None else layers
+        self.name = name
 
     def build(self,
               layer_norm_cond=None,
@@ -52,19 +54,15 @@ class Transformer(object):
         用来实现以“固定长度向量”为条件的条件Bert。
         """
         # Input
-        self.inputs = self.prepare_inputs()
-        outputs = self.inputs[:]
-        if additional_input_layers is not None:
-            if not isinstance(additional_input_layers, list):
-                additional_input_layers = [additional_input_layers]
-            self.inputs.extend(additional_input_layers)
+        inputs = self.prepare_inputs()
+        self.set_inputs(inputs, additional_input_layers)
+        outputs = inputs
         # Other
-        layer_norm_conds = [
+        self.layer_norm_conds = [
             layer_norm_cond,
             layer_norm_cond_hidden_size,
             layer_norm_cond_hidden_act or 'linear',
         ]
-        outputs.append(layer_norm_conds)
         # Embedding
         outputs = self.prepare_embeddings(outputs)
         # Main
@@ -72,9 +70,9 @@ class Transformer(object):
             outputs = self.prepare_main_layers(outputs, i)
         # Final
         outputs = self.prepare_final_layers(outputs)
-        self.outputs = outputs
+        self.set_outputs(outputs)
         # Model
-        self.model = keras.models.Model(self.inputs, self.outputs)
+        self.model = Model(self.inputs, self.outputs, name=self.name)
 
     def call(self, inputs, layer=None, arguments=None, **kwargs):
         """通过call调用层会自动重用同名层
@@ -117,6 +115,39 @@ class Transformer(object):
         """
         return self.position_bias
 
+    def set_inputs(self, inputs, additional_input_layers=None):
+        """设置input和inputs属性
+        """
+        if inputs is None:
+            inputs = []
+        elif not isinstance(inputs, list):
+            inputs = [inputs]
+
+        inputs = inputs[:]
+        if additional_input_layers is not None:
+            if not isinstance(additional_input_layers, list):
+                additional_input_layers = [additional_input_layers]
+            inputs.extend(additional_input_layers)
+
+        self.inputs = inputs
+        if len(inputs) > 1:
+            self.input = inputs
+        else:
+            self.input = inputs[0]
+
+    def set_outputs(self, outputs):
+        """设置output和oututs属性
+        """
+        if not isinstance(outputs, list):
+            outputs = [outputs]
+
+        outputs = outputs[:]
+        self.outputs = outputs
+        if len(outputs) > 1:
+            self.output = outputs
+        else:
+            self.output = outputs[0]
+
     @property
     def initializer(self):
         """默认使用截断正态分布初始化
@@ -206,8 +237,8 @@ class BERT(Transformer):
     def prepare_embeddings(self, inputs):
         """BERT的embedding是token、position、segment三者embedding之和
         """
-        x, s, layer_norm_conds = inputs
-        z = layer_norm_conds[0]
+        x, s = inputs
+        z = self.layer_norm_conds[0]
 
         x = self.call(inputs=x,
                       layer=Embedding,
@@ -233,8 +264,8 @@ class BERT(Transformer):
         x = self.call(inputs=self.simplify([x, z]),
                       layer=LayerNormalization,
                       conditional=(z is not None),
-                      hidden_units=layer_norm_conds[1],
-                      hidden_activation=layer_norm_conds[2],
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
                       hidden_initializer=self.initializer,
                       name='Embedding-Norm')
         x = self.call(inputs=x,
@@ -248,14 +279,14 @@ class BERT(Transformer):
                           kernel_initializer=self.initializer,
                           name='Embedding-Mapping')
 
-        return [x, layer_norm_conds]
+        return x
 
     def prepare_main_layers(self, inputs, index):
         """BERT的主体是基于Self-Attention的模块
         顺序：Att --> Add --> LN --> FFN --> Add --> LN
         """
-        x, layer_norm_conds = inputs
-        z = layer_norm_conds[0]
+        x = inputs
+        z = self.layer_norm_conds[0]
 
         attention_name = 'Transformer-%d-MultiHeadSelfAttention' % index
         feed_forward_name = 'Transformer-%d-FeedForward' % index
@@ -284,8 +315,8 @@ class BERT(Transformer):
         x = self.call(inputs=self.simplify([x, z]),
                       layer=LayerNormalization,
                       conditional=(z is not None),
-                      hidden_units=layer_norm_conds[1],
-                      hidden_activation=layer_norm_conds[2],
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
                       hidden_initializer=self.initializer,
                       name='%s-Norm' % attention_name)
 
@@ -307,18 +338,18 @@ class BERT(Transformer):
         x = self.call(inputs=self.simplify([x, z]),
                       layer=LayerNormalization,
                       conditional=(z is not None),
-                      hidden_units=layer_norm_conds[1],
-                      hidden_activation=layer_norm_conds[2],
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
                       hidden_initializer=self.initializer,
                       name='%s-Norm' % feed_forward_name)
 
-        return [x, layer_norm_conds]
+        return x
 
     def prepare_final_layers(self, inputs):
         """根据剩余参数决定输出
         """
-        x, layer_norm_conds = inputs
-        z = layer_norm_conds[0]
+        x = inputs
+        z = self.layer_norm_conds[0]
         outputs = [x]
 
         if self.with_pool or self.with_nsp:
@@ -357,8 +388,8 @@ class BERT(Transformer):
             x = self.call(inputs=self.simplify([x, z]),
                           layer=LayerNormalization,
                           conditional=(z is not None),
-                          hidden_units=layer_norm_conds[1],
-                          hidden_activation=layer_norm_conds[2],
+                          hidden_units=self.layer_norm_conds[1],
+                          hidden_activation=self.layer_norm_conds[2],
                           hidden_initializer=self.initializer,
                           name='MLM-Norm')
             mlm_activation = 'softmax' if self.with_mlm is True else self.with_mlm
@@ -477,8 +508,8 @@ class ALBERT(BERT):
         """ALBERT的主体是基于Self-Attention的模块
         顺序：Att --> Add --> LN --> FFN --> Add --> LN
         """
-        x, layer_norm_conds = inputs
-        z = layer_norm_conds[0]
+        x = inputs
+        z = self.layer_norm_conds[0]
 
         attention_name = 'Transformer-MultiHeadSelfAttention'
         feed_forward_name = 'Transformer-FeedForward'
@@ -507,8 +538,8 @@ class ALBERT(BERT):
         x = self.call(inputs=self.simplify([x, z]),
                       layer=LayerNormalization,
                       conditional=(z is not None),
-                      hidden_units=layer_norm_conds[1],
-                      hidden_activation=layer_norm_conds[2],
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
                       hidden_initializer=self.initializer,
                       name='%s-Norm' % attention_name)
 
@@ -530,12 +561,12 @@ class ALBERT(BERT):
         x = self.call(inputs=self.simplify([x, z]),
                       layer=LayerNormalization,
                       conditional=(z is not None),
-                      hidden_units=layer_norm_conds[1],
-                      hidden_activation=layer_norm_conds[2],
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
                       hidden_initializer=self.initializer,
                       name='%s-Norm' % feed_forward_name)
 
-        return [x, layer_norm_conds]
+        return x
 
     def variable_mapping(self):
         """映射到官方ALBERT权重格式
@@ -624,8 +655,8 @@ class NEZHA(BERT):
     def prepare_embeddings(self, inputs):
         """NEZHA的embedding是token、segment两者embedding之和
         """
-        x, s, layer_norm_conds = inputs
-        z = layer_norm_conds[0]
+        x, s = inputs
+        z = self.layer_norm_conds[0]
 
         x = self.call(inputs=x,
                       layer=Embedding,
@@ -644,8 +675,8 @@ class NEZHA(BERT):
         x = self.call(inputs=self.simplify([x, z]),
                       layer=LayerNormalization,
                       conditional=(z is not None),
-                      hidden_units=layer_norm_conds[1],
-                      hidden_activation=layer_norm_conds[2],
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
                       hidden_initializer=self.initializer,
                       name='Embedding-Norm')
         x = self.call(inputs=x,
@@ -659,14 +690,14 @@ class NEZHA(BERT):
                           kernel_initializer=self.initializer,
                           name='Embedding-Mapping')
 
-        return [x, layer_norm_conds]
+        return x
 
     def prepare_main_layers(self, inputs, index):
         """NEZHA的主体是基于Self-Attention的模块
         顺序：Att --> Add --> LN --> FFN --> Add --> LN
         """
-        x, layer_norm_conds = inputs
-        z = layer_norm_conds[0]
+        x = inputs
+        z = self.layer_norm_conds[0]
 
         attention_name = 'Transformer-%d-MultiHeadSelfAttention' % index
         feed_forward_name = 'Transformer-%d-FeedForward' % index
@@ -697,8 +728,8 @@ class NEZHA(BERT):
         x = self.call(inputs=self.simplify([x, z]),
                       layer=LayerNormalization,
                       conditional=(z is not None),
-                      hidden_units=layer_norm_conds[1],
-                      hidden_activation=layer_norm_conds[2],
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
                       hidden_initializer=self.initializer,
                       name='%s-Norm' % attention_name)
 
@@ -720,12 +751,12 @@ class NEZHA(BERT):
         x = self.call(inputs=self.simplify([x, z]),
                       layer=LayerNormalization,
                       conditional=(z is not None),
-                      hidden_units=layer_norm_conds[1],
-                      hidden_activation=layer_norm_conds[2],
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
                       hidden_initializer=self.initializer,
                       name='%s-Norm' % feed_forward_name)
 
-        return [x, layer_norm_conds]
+        return x
 
     def compute_position_bias(self, inputs=None):
         """经典相对位置编码
@@ -756,6 +787,219 @@ class NEZHA(BERT):
         return self.position_bias
 
 
+class GPT2_ML(Transformer):
+    """构建GPT2_ML模型
+    链接: https://github.com/imcaspar/gpt2-ml
+    """
+    def __init__(
+            self,
+            max_position,  # 序列最大长度
+            final_activation='softmax',  # 预测分布的激活函数
+            **kwargs  # 其余参数
+    ):
+        super(GPT2_ML, self).__init__(**kwargs)
+        self.max_position = max_position
+        self.final_activation = final_activation
+
+    def prepare_inputs(self):
+        """GPT2_ML的输入是token_ids和segment_ids
+        """
+        x_in = Input(shape=(None, ), name='Input-Token')
+        return x_in
+
+    def prepare_embeddings(self, inputs):
+        """GPT2_ML的embedding是token、position两者embedding之和
+        """
+        x = inputs
+        z = self.layer_norm_conds[0]
+
+        x = self.call(inputs=x,
+                      layer=Embedding,
+                      input_dim=self.vocab_size,
+                      output_dim=self.embedding_size,
+                      embeddings_initializer=self.initializer,
+                      mask_zero=True,
+                      name='Embedding-Token')
+        x = self.call(inputs=x,
+                      layer=PositionEmbedding,
+                      input_dim=self.max_position,
+                      output_dim=self.embedding_size,
+                      merge_mode='add',
+                      embeddings_initializer=self.initializer,
+                      name='Embedding-Position')
+        x = self.call(inputs=self.simplify([x, z]),
+                      layer=LayerNormalization,
+                      epsilon=1e-5,
+                      conditional=(z is not None),
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
+                      hidden_initializer=self.initializer,
+                      name='Embedding-Norm')
+        if self.embedding_size != self.hidden_size:
+            x = self.call(inputs=x,
+                          layer=Dense,
+                          units=self.hidden_size,
+                          kernel_initializer=self.initializer,
+                          name='Embedding-Mapping')
+
+        return x
+
+    def prepare_main_layers(self, inputs, index):
+        """GPT2_ML的主体是基于Self-Attention的模块
+        顺序：Att  --> LN --> FFN --> Add --> LN
+        """
+        x = inputs
+        z = self.layer_norm_conds[0]
+
+        attention_name = 'Transformer-%d-MultiHeadSelfAttention' % index
+        feed_forward_name = 'Transformer-%d-FeedForward' % index
+        attention_mask = self.compute_attention_mask()
+
+        # Self Attention
+        xi, x, arguments = x, [x, x, x, attention_mask], {'a_mask': True}
+
+        x = self.call(inputs=x,
+                      layer=MultiHeadAttention,
+                      arguments=arguments,
+                      heads=self.num_attention_heads,
+                      head_size=self.attention_head_size,
+                      kernel_initializer=self.initializer,
+                      name=attention_name)
+        x = self.call(inputs=x,
+                      layer=Dropout,
+                      rate=self.dropout_rate,
+                      name='%s-Dropout' % attention_name)
+        x = self.call(inputs=[xi, x],
+                      layer=Add,
+                      name='%s-Add' % attention_name)
+
+        # Feed Forward
+        xi = x
+        x = self.call(inputs=self.simplify([x, z]),
+                      layer=LayerNormalization,
+                      epsilon=1e-5,
+                      conditional=(z is not None),
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
+                      hidden_initializer=self.initializer,
+                      name='%s-Norm-0' % feed_forward_name)
+        x = self.call(inputs=x,
+                      layer=FeedForward,
+                      units=self.intermediate_size,
+                      activation=self.hidden_act,
+                      kernel_initializer=self.initializer,
+                      name=feed_forward_name)
+        x = self.call(inputs=x,
+                      layer=Dropout,
+                      rate=self.dropout_rate,
+                      name='%s-Dropout' % feed_forward_name)
+        x = self.call(inputs=[xi, x],
+                      layer=Add,
+                      name='%s-Add' % feed_forward_name)
+        x = self.call(inputs=self.simplify([x, z]),
+                      layer=LayerNormalization,
+                      epsilon=1e-5,
+                      conditional=(z is not None),
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
+                      hidden_initializer=self.initializer,
+                      name='%s-Norm-1' % feed_forward_name)
+
+        return x
+
+    def prepare_final_layers(self, inputs):
+        """剩余部分
+        """
+        x = inputs
+        z = self.layer_norm_conds[0]
+
+        # Language Model部分
+        x = self.call(inputs=x,
+                      layer=EmbeddingDense,
+                      embedding_name='Embedding-Token',
+                      activation=self.final_activation,
+                      name='LM-Proba')
+
+        return x
+
+    def load_variable(self, checkpoint, name):
+        """加载单个变量的函数
+        """
+        variable = super(GPT2_ML, self).load_variable(checkpoint, name)
+        if name == 'newslm/embeddings/word_embed':
+            if self.keep_tokens is None:
+                return variable
+            else:
+                return variable[self.keep_tokens]
+        else:
+            return variable
+
+    def compute_attention_mask(self, inputs=None):
+        """添加下三角形式的attention mask
+        """
+        if self.attention_mask is None:
+
+            def lm_mask(s):
+                import tensorflow as tf
+                seq_len = K.shape(s)[1]
+                with K.name_scope('attention_mask'):
+                    ones = K.ones((1, 1, seq_len, seq_len))
+                a_mask = tf.linalg.band_part(ones, -1, 0)
+                return a_mask
+
+            self.attention_mask = self.call(inputs=self.inputs[0],
+                                            layer=Lambda,
+                                            function=lm_mask,
+                                            name='Attention-LM-Mask')
+
+        return self.attention_mask
+
+    def variable_mapping(self):
+        """映射到官方GPT2_ML权重格式
+        """
+        mapping = {
+            'Embedding-Token': ['newslm/embeddings/word_embed'],
+            'Embedding-Position': ['newslm/embeddings/pos_embed'],
+            'Embedding-Norm': [
+                'newslm/embeddings/LayerNorm_embed_norm/beta',
+                'newslm/embeddings/LayerNorm_embed_norm/gamma',
+            ],
+        }
+
+        for i in range(self.num_hidden_layers):
+            prefix = 'newslm/layer%02d/' % i
+            mapping.update({
+                'Transformer-%d-MultiHeadSelfAttention' % i: [
+                    prefix + 'query_layer/kernel',
+                    prefix + 'query_layer/bias',
+                    prefix + 'key_layer/kernel',
+                    prefix + 'key_layer/bias',
+                    prefix + 'value_layer/kernel',
+                    prefix + 'value_layer/bias',
+                    prefix + 'context_projection_layer/kernel',
+                    prefix + 'context_projection_layer/bias',
+                ],
+                'Transformer-%d-FeedForward-Norm-0' % i: [
+                    prefix + 'LayerNorm_mlp_ln0/beta',
+                    prefix + 'LayerNorm_mlp_ln0/gamma',
+                ],
+                'Transformer-%d-FeedForward' % i: [
+                    prefix + 'intermediate/kernel',
+                    prefix + 'intermediate/bias',
+                    prefix + 'output/kernel',
+                    prefix + 'output/bias',
+                ],
+                'Transformer-%d-FeedForward-Norm-1' % i: [
+                    prefix + 'LayerNorm_mlp_ln1/beta',
+                    prefix + 'LayerNorm_mlp_ln1/gamma',
+                ],
+            })
+
+        mapping = {k: v for k, v in mapping.items() if k in self.layers}
+
+        return mapping
+
+
 class T5_Base(Transformer):
     """Google的T5模型（基类）
     """
@@ -858,13 +1102,13 @@ class T5_Encoder(T5_Base):
         """T5的Encoder的输入只有token_ids
         """
         x_in = Input(shape=(None, ), name='Encoder-Input-Token')
-        return [x_in]
+        return x_in
 
     def prepare_embeddings(self, inputs):
         """T5的embedding只有token embedding，
         并把relative position embedding准备好，待attention使用。
         """
-        x, layer_norm_conds = inputs
+        x = inputs
 
         x = self.call(inputs=x,
                       layer=Embedding,
@@ -884,15 +1128,15 @@ class T5_Encoder(T5_Base):
                           kernel_initializer=self.initializer,
                           name='Encoder-Embedding-Mapping')
 
-        return [x, layer_norm_conds]
+        return x
 
     def prepare_main_layers(self, inputs, index):
         """T5的Encoder的主体是基于Self-Attention的模块
         顺序：LN --> Att --> Add --> LN --> FFN --> Add
         """
 
-        x, layer_norm_conds = inputs
-        z = layer_norm_conds[0]
+        x = inputs
+        z = self.layer_norm_conds[0]
 
         attention_name = 'Encoder-Transformer-%d-MultiHeadSelfAttention' % index
         feed_forward_name = 'Encoder-Transformer-%d-FeedForward' % index
@@ -904,9 +1148,10 @@ class T5_Encoder(T5_Base):
         x = self.call(inputs=self.simplify([x, z]),
                       layer=LayerNormalization,
                       center=False,
+                      epsilon=1e-6,
                       conditional=(z is not None),
-                      hidden_units=layer_norm_conds[1],
-                      hidden_activation=layer_norm_conds[2],
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
                       hidden_initializer=self.initializer,
                       name='%s-Norm' % attention_name)
         x = self.call(inputs=[x, x, x, position_bias],
@@ -930,9 +1175,10 @@ class T5_Encoder(T5_Base):
         x = self.call(inputs=self.simplify([x, z]),
                       layer=LayerNormalization,
                       center=False,
+                      epsilon=1e-6,
                       conditional=(z is not None),
-                      hidden_units=layer_norm_conds[1],
-                      hidden_activation=layer_norm_conds[2],
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
                       hidden_initializer=self.initializer,
                       name='%s-Norm' % feed_forward_name)
         x = self.call(inputs=x,
@@ -950,20 +1196,21 @@ class T5_Encoder(T5_Base):
                       layer=Add,
                       name='%s-Add' % feed_forward_name)
 
-        return [x, layer_norm_conds]
+        return x
 
     def prepare_final_layers(self, inputs):
         """剩余部分
         """
-        x, layer_norm_conds = inputs
-        z = layer_norm_conds[0]
+        x = inputs
+        z = self.layer_norm_conds[0]
 
         x = self.call(inputs=self.simplify([x, z]),
                       layer=LayerNormalization,
                       center=False,
+                      epsilon=1e-6,
                       conditional=(z is not None),
-                      hidden_units=layer_norm_conds[1],
-                      hidden_activation=layer_norm_conds[2],
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
                       hidden_initializer=self.initializer,
                       name='Encoder-Output-Norm')
         x = self.call(inputs=x,
@@ -1012,7 +1259,7 @@ class T5_Decoder(Transformer):
         """T5的embedding只有token embedding，
         并把relative position embedding准备好，待attention使用。
         """
-        c, x, layer_norm_conds = inputs
+        c, x = inputs
 
         x = self.call(inputs=x,
                       layer=Embedding,
@@ -1032,15 +1279,15 @@ class T5_Decoder(Transformer):
                           kernel_initializer=self.initializer,
                           name='Decoder-Embedding-Mapping')
 
-        return [c, x, layer_norm_conds]
+        return [c, x]
 
     def prepare_main_layers(self, inputs, index):
         """T5的Dencoder主体是基于Self-Attention、Cross-Attention的模块
         顺序：LN --> Att1 --> Add --> LN --> Att2 --> Add -->  LN --> FFN --> Add
         """
 
-        c, x, layer_norm_conds = inputs
-        z = layer_norm_conds[0]
+        c, x = inputs
+        z = self.layer_norm_conds[0]
 
         self_attention_name = 'Decoder-Transformer-%d-MultiHeadSelfAttention' % index
         cross_attention_name = 'Decoder-Transformer-%d-MultiHeadCrossAttention' % index
@@ -1053,9 +1300,10 @@ class T5_Decoder(Transformer):
         x = self.call(inputs=self.simplify([x, z]),
                       layer=LayerNormalization,
                       center=False,
+                      epsilon=1e-6,
                       conditional=(z is not None),
-                      hidden_units=layer_norm_conds[1],
-                      hidden_activation=layer_norm_conds[2],
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
                       hidden_initializer=self.initializer,
                       name='%s-Norm' % self_attention_name)
         x = self.call(inputs=[x, x, x, attention_mask, position_bias[0]],
@@ -1079,9 +1327,10 @@ class T5_Decoder(Transformer):
         x = self.call(inputs=self.simplify([x, z]),
                       layer=LayerNormalization,
                       center=False,
+                      epsilon=1e-6,
                       conditional=(z is not None),
-                      hidden_units=layer_norm_conds[1],
-                      hidden_activation=layer_norm_conds[2],
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
                       hidden_initializer=self.initializer,
                       name='%s-Norm' % cross_attention_name)
         x = self.call(inputs=[x, c, c, position_bias[1]],
@@ -1105,9 +1354,10 @@ class T5_Decoder(Transformer):
         x = self.call(inputs=self.simplify([x, z]),
                       layer=LayerNormalization,
                       center=False,
+                      epsilon=1e-6,
                       conditional=(z is not None),
-                      hidden_units=layer_norm_conds[1],
-                      hidden_activation=layer_norm_conds[2],
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
                       hidden_initializer=self.initializer,
                       name='%s-Norm' % feed_forward_name)
         x = self.call(inputs=x,
@@ -1125,20 +1375,21 @@ class T5_Decoder(Transformer):
                       layer=Add,
                       name='%s-Add' % feed_forward_name)
 
-        return [c, x, layer_norm_conds]
+        return [c, x]
 
     def prepare_final_layers(self, inputs):
         """剩余部分
         """
-        c, x, layer_norm_conds = inputs
-        z = layer_norm_conds[0]
+        c, x = inputs
+        z = self.layer_norm_conds[0]
 
         x = self.call(inputs=self.simplify([x, z]),
                       layer=LayerNormalization,
                       center=False,
+                      epsilon=1e-6,
                       conditional=(z is not None),
-                      hidden_units=layer_norm_conds[1],
-                      hidden_activation=layer_norm_conds[2],
+                      hidden_units=self.layer_norm_conds[1],
+                      hidden_activation=self.layer_norm_conds[2],
                       hidden_initializer=self.initializer,
                       name='Decoder-Output-Norm')
         x = self.call(inputs=x,
@@ -1218,8 +1469,13 @@ class T5(T5_Base):
     def __init__(self, **kwargs):
         super(T5, self).__init__(**kwargs)
         kwargs['layers'] = self.layers
-        self._encoder = T5_Encoder(**kwargs)
-        self._decoder = T5_Decoder(**kwargs)
+        e_name, d_name = 'Encoder', 'Decoder'
+        if 'name' in kwargs:
+            e_name = '%s_%s' % (kwargs['name'], e_name)
+            d_name = '%s_%s' % (kwargs['name'], d_name)
+            del kwargs['name']  # 防止重复传参
+        self._encoder = T5_Encoder(name=e_name, **kwargs)
+        self._decoder = T5_Decoder(name=d_name, **kwargs)
 
     def build(self, **kwargs):
         """同时构建Encoder和Decoder
@@ -1326,6 +1582,7 @@ def build_transformer_model(config_path=None,
         'albert': ALBERT,
         'albert_unshared': ALBERT_Unshared,
         'nezha': NEZHA,
+        'gpt2_ml': GPT2_ML,
         't5': T5,
     }
     MODEL = models[model]
@@ -1349,13 +1606,10 @@ def build_transformer_model(config_path=None,
 
 
 def build_bert_model(*args, **kwargs):
-    warnings.warn(
-        'build_bert_model has been renamed as build_transformer_model.')
+    warnings.warn('build_bert_model has been renamed as build_transformer_model.')
     warnings.warn('please use build_transformer_model.')
     if kwargs.get('application') == 'seq2seq':
-        warnings.warn(
-            'application=\'seq2seq\' has been renamed as application=\'unilm\''
-        )
+        warnings.warn('application=\'seq2seq\' has been renamed as application=\'unilm\'')
         warnings.warn('please use application=\'unilm\'')
         kwargs['application'] = 'unilm'
     return build_transformer_model(*args, **kwargs)
