@@ -235,13 +235,38 @@ class DataGenerator(object):
     def __init__(self, data, batch_size=32):
         self.data = data
         self.batch_size = batch_size
-        self.steps = len(self.data) // self.batch_size
-        if len(self.data) % self.batch_size != 0:
-            self.steps += 1
+        if hasattr(self.data, '__len__'):
+            self.steps = len(self.data) // self.batch_size
+            if len(self.data) % self.batch_size != 0:
+                self.steps += 1
+        else:
+            self.steps = None
 
     def __len__(self):
         return self.steps
 
+    def sample(self, random=False):
+        """采样函数，每个样本同时返回一个is_end标记
+        """
+        if random and self.steps is not None:
+
+            def generator():
+                indices = list(range(len(self.data)))
+                np.random.shuffle(indices)
+                for i in indices:
+                    yield self.data[i]
+
+            data = generator()
+        else:
+            data = iter(self.data)
+
+        d_current = next(data)
+        for d_next in data:
+            yield False, d_current
+            d_current = d_next
+
+        yield True, d_current
+
     def __iter__(self, random=False):
         raise NotImplementedError
 
@@ -263,10 +288,11 @@ class AutoRegressiveDecoder(object):
     """通用自回归生成模型解码基类
     包含beam search和random sample两种策略
     """
-    def __init__(self, start_id, end_id, maxlen):
+    def __init__(self, start_id, end_id, maxlen, minlen=None):
         self.start_id = start_id
         self.end_id = end_id
         self.maxlen = maxlen
+        self.minlen = minlen or 1
         if start_id is None:
             self.first_output_ids = np.empty((1, 0), dtype=int)
         else:
@@ -320,22 +346,24 @@ class AutoRegressiveDecoder(object):
             indices_2 = (indices % scores.shape[1]).reshape((-1, 1))  # 列索引
             output_ids = np.concatenate([output_ids[indices_1], indices_2], 1)  # 更新输出
             output_scores = np.take_along_axis(scores, indices, axis=None)  # 更新得分
-            best_one = output_scores.argmax()  # 得分最大的那个
-            if indices_2[best_one, 0] == self.end_id:  # 如果已经终止
-                return output_ids[best_one]  # 直接输出
-            else:  # 否则，只保留未完成部分
-                flag = (indices_2[:, 0] != self.end_id)  # 标记未完成序列
-                if not flag.all():  # 如果有已完成的
-                    inputs = [i[flag] for i in inputs]  # 扔掉已完成序列
-                    output_ids = output_ids[flag]  # 扔掉已完成序列
-                    output_scores = output_scores[flag]  # 扔掉已完成序列
-                    topk = flag.sum()  # topk相应变化
+            if output_ids.shape[1] >= self.minlen:  # 最短长度判断
+                best_one = output_scores.argmax()  # 得分最大的那个
+                if indices_2[best_one, 0] == self.end_id:  # 如果已经终止
+                    return output_ids[best_one]  # 直接输出
+                else:  # 否则，只保留未完成部分
+                    flag = (indices_2[:, 0] != self.end_id)  # 标记未完成序列
+                    if not flag.all():  # 如果有已完成的
+                        inputs = [i[flag] for i in inputs]  # 扔掉已完成序列
+                        output_ids = output_ids[flag]  # 扔掉已完成序列
+                        output_scores = output_scores[flag]  # 扔掉已完成序列
+                        topk = flag.sum()  # topk相应变化
         # 达到长度直接输出
         return output_ids[output_scores.argmax()]
 
-    def random_sample(self, inputs, n, topk=None):
+    def random_sample(self, inputs, n, topk=None, topp=None):
         """随机采样n个结果
-        说明：非None的topk表示每一步只从概率最高的topk个中采样；
+        说明：非None的topk表示每一步只从概率最高的topk个中采样；而非None的topp
+             表示每一步只从概率最高的且概率之和刚好达到topp的若干个token中采样。
         返回：n个解码序列组成的list。
         """
         inputs = [np.array([i]) for i in inputs]
@@ -343,29 +371,41 @@ class AutoRegressiveDecoder(object):
         results = []
         for step in range(self.maxlen):
             probas = self.predict(inputs, output_ids, step, 'probas')  # 计算当前概率
+            probas /= probas.sum(axis=1, keepdims=True)  # 确保归一化
             if step == 0:  # 第1步预测后将结果重复n次
                 probas = np.repeat(probas, n, axis=0)
                 inputs = [np.repeat(i, n, axis=0) for i in inputs]
                 output_ids = np.repeat(output_ids, n, axis=0)
             if topk is not None:
-                indices = probas.argpartition(-topk, axis=1)[:, -topk:]  # 仅保留topk
-                probas = np.take_along_axis(probas, indices, axis=1)  # topk概率
-            probas /= probas.sum(axis=1, keepdims=True)  # 重新归一化
+                k_indices = probas.argpartition(-topk, axis=1)[:, -topk:]  # 仅保留topk
+                probas = np.take_along_axis(probas, k_indices, axis=1)  # topk概率
+                probas /= probas.sum(axis=1, keepdims=True)  # 重新归一化
+            if topp is not None:
+                p_indices = probas.argsort(axis=1)[:, ::-1]  # 从高到低排序
+                probas = np.take_along_axis(probas, p_indices, axis=1)  # 排序概率
+                cumsum_probas = np.cumsum(probas, axis=1)  # 累积概率
+                flag = np.roll(cumsum_probas >= topp, 1, axis=1)  # 标记超过topp的部分
+                flag[:, 0] = False  # 结合上面的np.roll，实现平移一位的效果
+                probas[flag] = 0  # 后面的全部置零
+                probas /= probas.sum(axis=1, keepdims=True)  # 重新归一化
             sample_func = lambda p: np.random.choice(len(p), p=p)  # 按概率采样函数
             sample_ids = np.apply_along_axis(sample_func, 1, probas)  # 执行采样
             sample_ids = sample_ids.reshape((-1, 1))  # 对齐形状
+            if topp is not None:
+                sample_ids = np.take_along_axis(p_indices, sample_ids, axis=1)  # 对齐原id
             if topk is not None:
-                sample_ids = np.take_along_axis(indices, sample_ids, axis=1)  # 对齐原id
+                sample_ids = np.take_along_axis(k_indices, sample_ids, axis=1)  # 对齐原id
             output_ids = np.concatenate([output_ids, sample_ids], 1)  # 更新输出
-            flag = (sample_ids[:, 0] == self.end_id)  # 标记已完成序列
-            if flag.any():  # 如果有已完成的
-                for ids in output_ids[flag]:  # 存好已完成序列
-                    results.append(ids)
-                flag = (flag == False)  # 标记未完成序列
-                inputs = [i[flag] for i in inputs]  # 只保留未完成部分输入
-                output_ids = output_ids[flag]  # 只保留未完成部分候选集
-                if len(output_ids) == 0:
-                    break
+            if output_ids.shape[1] >= self.minlen:  # 最短长度判断
+                flag = (sample_ids[:, 0] == self.end_id)  # 标记已完成序列
+                if flag.any():  # 如果有已完成的
+                    for ids in output_ids[flag]:  # 存好已完成序列
+                        results.append(ids)
+                    flag = (flag == False)  # 标记未完成序列
+                    inputs = [i[flag] for i in inputs]  # 只保留未完成部分输入
+                    output_ids = output_ids[flag]  # 只保留未完成部分候选集
+                    if len(output_ids) == 0:
+                        break
         # 如果还有未完成序列，直接放入结果
         for ids in output_ids:
             results.append(ids)
